#include "DefaultStyle.h"
#include <QStyleFactory>
#include <QStyle>
#include <QSettings>
#include "globals.h"
#include "stringmacros.h"

DefaultStyle::DefaultStyle(int    argc,
                           char*  argv[])
   : seenStyle(false)
{
   // It seems to be impossible to distinguish directly whether or not a user
   // specified a style or whether the style in use is the ultimate Qt
   // fallback.  All the relevant globals are private or static, and none of
   // the relevant functions are virtual.  So, the only way to determine this
   // is by snooping the command line and the Qt resource settings.  And it
   // must be done early before creating a QApplication object which would
   // remove the options from the command line.  This is ugly, but it was a
   // choice between ugly and impossible.

   int  arg = 1;
   for (; arg < argc; arg++) {
      if (   strncmp(argv[arg], "-style=", STRLEN("-style=")) == 0
          || strcmp(argv[arg], "-style") == 0) {
         seenStyle = true;
         return;
      }
   }

   QSettings settings(QSettings::UserScope, "Trolltech");
   settings.beginGroup("Qt");
   if (settings.contains("style")) {
      seenStyle = true;
   }
}

void DefaultStyle::set(QApplication&  app,
                       QString        name)
{
   if (!seenStyle) {
      QStyle*  style = QStyleFactory::create(name);
      if (style == NULL) {
         fprintf(stderr, "%s: unrecognized style: %s\n",
                 program, name.toStdString().c_str());
         fprintf(stderr, "%s: known styles:\n", program);
         QStringList  keys = QStyleFactory::keys();
         QStringList::const_iterator  it = keys.begin();
         for (; it != keys.end(); ++it) {
            fprintf(stderr, "%s:    %s\n", program, it->toStdString().c_str());
         }
         exit(1);
      }
      app.setStyle(style);
      app.setPalette(style->standardPalette());
   }
}
